@@ -7,38 +7,63 @@ seg.glm.fit.boot<-function(y, XREG, Z, PSI, w, offs, opz, n.boot=10, size.boot=N
 #show.history() se c'e' stato boot restart potrebbe produrre un grafico 2x1 di "dev vs it" and "no.of distinct vs it"
 #--------
       extract.psi<-function(lista){
-      #serve per estrarre il miglior psi..
-            dev.values<-lista[[1]][-1] #remove the 1st one referring to model without psi
-            psi.values<-lista[[2]][-1] #remove the 1st one (NA)
-            dev.ok<-min(dev.values)
-          	id.dev.ok<-which.min(dev.values)
-          	if(is.list(psi.values))  psi.values<-matrix(unlist(psi.values),
-          		nrow=length(dev.values), byrow=TRUE)
-          	if(!is.matrix(psi.values)) psi.values<-matrix(psi.values)
-          	psi.ok<-psi.values[id.dev.ok,]
-          	r<-list(dev.no.gap=dev.ok, psi=psi.ok)
-          	r
-      	}
+        #serve per estrarre il miglior psi..
+        dev.values<-lista[[1]]
+        psi.values<-lista[[2]]
+        if(any(is.na(psi.values[[1]]))) {#se la 1 componente e' NA (fino alla versione 2.0-3 era cosi'... perche' in dev.values c'erano 
+          #  anche i valori relativi al modello senza psi.. )
+          dev.values<-dev.values[-1] #remove the 1st one referring to model without psi
+          psi.values<-psi.values[-1]
+        }
+        dev.ok<-min(dev.values)
+        id.dev.ok<-which.min(dev.values)
+        if(is.list(psi.values))  psi.values<-matrix(unlist(psi.values),
+                                                    nrow=length(dev.values), byrow=TRUE)
+        if(!is.matrix(psi.values)) psi.values<-matrix(psi.values)
+        psi.ok<-psi.values[id.dev.ok,]
+        r<-list(dev.no.gap=dev.ok, psi=psi.ok)
+        r
+      }
       #-------------
+      if(is.null(opz$seed)){
+        mY <- mean(y)
+        sepDec<-if(options()$OutDec==".") "\\." else "\\,"
+        vv <- strsplit(paste(strsplit(paste(mY), sepDec)[[1]], collapse=""),"")[[1]]
+        vv<-vv[vv!="0"]
+        vv=na.omit(vv[1:5])
+        seed <-eval(parse(text=paste(vv, collapse="")))
+        set.seed(seed)
+      } else {
+        if(is.na(opz$seed)) {
+          seed <-eval(parse(text=paste(sample(0:9, size=6), collapse="")))
+          set.seed(seed)
+        } else {
+          seed <-opz$seed
+          set.seed(opz$seed)
+        }
+      }  
+
       if(!nonParam){
         nonParam<-TRUE
         warning("`nonParam' set to TRUE for segmented glm..", call.=FALSE)
       } 
       visualBoot<-opz$visualBoot
-      opz.boot<-opz
-      opz.boot$pow=c(1,1) #c(1.1,1.2)
+      #opz.boot<-opz
+      #opz.boot$pow=c(1,1) #c(1.1,1.2)
       opz1<-opz
-      opz1$it.max <-1
+      opz1$it.max <- 0 #opz1 viene usata solo quando diversi tentativi di stimare il modello falliscono...
       n<-length(y)
-      o0<-try(suppressWarnings(seg.glm.fit(y, XREG, Z, PSI, w, offs, opz)), silent=TRUE)
-      rangeZ <- apply(Z, 2, range) #serve sempre 
-      
       alpha<-opz$alpha
       #limZ <- apply(Z, 2, quantile, names=FALSE, probs=c(alpha,1-alpha))
       limZ <- apply(Z, 2, quantile, names = FALSE, probs = c(alpha[1], alpha[2]))
+      rangeZ <- apply(Z, 2, range) #serve sempre 
       
-      #browser()
-      
+      #13/03/24:
+      #In realta' non ci sarebbe neanche bisogno di questo fit:... Comunque, facciamolo in economia..
+      opz0 <- opz
+      opz0$maxit.glm <- 2
+      o0<-try(suppressWarnings(seg.glm.fit(y, XREG, Z, PSI, w, offs, opz0)), silent=TRUE)
+           
       if(!is.list(o0)) {
         o0<- suppressWarnings(seg.glm.fit(y, XREG, Z, PSI, w, offs, opz, return.all.sol=TRUE))
         o0<-extract.psi(o0)
@@ -47,20 +72,23 @@ seg.glm.fit.boot<-function(y, XREG, Z, PSI, w, offs, opz, n.boot=10, size.boot=N
       if(is.list(o0)){
         est.psi00<-est.psi0<-o0$psi
         ss00<-o0$dev.no.gap
+        eta0 <- o0$eta0
         if(!nonParam) fitted.ok<-fitted(o0)
       } else {
-        if(!nonParam) stop("semiparametric boot requires reasonable fitted values. try a different psi or use nonparam boot")
+        if(!nonParam) stop("semiparametric boot requires reasonable fitted values. Try a different psi or use nonparam boot")
         if(random) {
           est.psi00<-est.psi0<-apply(limZ,2,function(r)runif(1,r[1],r[2]))
-          PSI1 <- matrix(rep(est.psi0, rep(nrow(Z), length(est.psi0))), ncol = length(est.psi0))
+          PSI1 <- matrix(est.psi0, n, ncol = length(est.psi0), byrow=TRUE)
           o0<-try(suppressWarnings(seg.glm.fit(y, XREG, Z, PSI1, w, offs, opz1)), silent=TRUE)
           ss00<-o0$dev.no.gap
+          eta0 <- o0$eta0
         } else {
           est.psi00<-est.psi0<-apply(PSI,2,mean)
           ss00<-opz$dev0
+          eta0 <- NULL
         }
       }
-      n.intDev0<-nchar(strsplit(as.character(ss00),"\\.")[[1]][1])
+      n.intDev0<-nchar(strsplit(as.character(ss00), options()$OutDec)[[1]][1])
       
       all.est.psi.boot<-all.selected.psi<-all.est.psi<-matrix(, nrow=n.boot, ncol=length(est.psi0))
       all.ss<-all.selected.ss<-rep(NA, n.boot)
@@ -70,7 +98,9 @@ seg.glm.fit.boot<-function(y, XREG, Z, PSI, w, offs, opz, n.boot=10, size.boot=N
 #      if(visualBoot) cat(0, " ", formatC(opz$dev0, 3, format = "f"),"", "(No breakpoint(s))", "\n")
       count.random<-0
       alpha<-.1
+      #=====================================
       for(k in seq(n.boot)){
+        opz$eta0 <- eta0
         ##se gli *ultimi* n.boot.rev valori di ss sono uguali, cambia i psi...
         n.boot.rev<- 3 #3 o 4?
         diff.selected.ss <- rev(diff(na.omit(all.selected.ss)))
@@ -81,7 +111,7 @@ seg.glm.fit.boot<-function(y, XREG, Z, PSI, w, offs, opz, n.boot=10, size.boot=N
           alpha<- 1-alpha
           est.psi0<-sapply(1:ncol(Z),function(i)quantile(Z[,i],probs=1-qpsi[i],names=FALSE))
         }
-          PSI <- matrix(rep(est.psi0, rep(nrow(Z), length(est.psi0))), ncol = length(est.psi0))
+          PSI <- matrix(est.psi0, n, ncol = length(est.psi0), byrow=TRUE)
           if(jt) Z<-apply(Z.orig,2,jitter)
           if(nonParam){
             id<-sample(n, size=size.boot, replace=TRUE)
@@ -96,13 +126,17 @@ seg.glm.fit.boot<-function(y, XREG, Z, PSI, w, offs, opz, n.boot=10, size.boot=N
           } else {
             est.psi.boot<-apply(limZ,2,function(r)runif(1,r[1],r[2]))
           }
-          PSI <- matrix(rep(est.psi.boot, rep(nrow(Z), length(est.psi.boot))), ncol = length(est.psi.boot))
-          opz$h<-max(opz$h*.9, .2)
-          opz$it.max<-opz$it.max+1
+          PSI <- matrix(est.psi.boot, n, ncol = length(est.psi.boot), byrow=TRUE)
+          #opz$h<-max(opz$h*.9, .2)
+          #opz$it.max<-opz$it.max+1
+          
+          opz$Nboot <- k
+          
           o<-try(suppressWarnings(seg.glm.fit(y, XREG, Z.orig, PSI, w, offs, opz, return.all.sol=TRUE)), silent=TRUE)
+
           if(!is.list(o) && random){
             est.psi00<-est.psi0<-apply(limZ,2,function(r)runif(1,r[1],r[2]))
-            PSI1 <- matrix(rep(est.psi0, rep(nrow(Z), length(est.psi0))), ncol = length(est.psi0))
+            PSI1 <- matrix(est.psi0, n, ncol = length(est.psi0), byrow=TRUE)
             o<-try(suppressWarnings(seg.glm.fit(y, XREG, Z, PSI1, w, offs, opz1)), silent=TRUE)
             count.random<-count.random+1
           }
@@ -114,6 +148,7 @@ seg.glm.fit.boot<-function(y, XREG, Z, PSI, w, offs, opz, n.boot=10, size.boot=N
             est.psi0<-o0$psi
             all.selected.psi[k,] <- est.psi0
             all.selected.ss[k]<-o0$dev.no.gap #min(c(o$SumSquares.no.gap, o0$SumSquares.no.gap))
+            eta0 <- o0$eta0
           }
 
           if (visualBoot) {
@@ -135,12 +170,53 @@ seg.glm.fit.boot<-function(y, XREG, Z, PSI, w, offs, opz, n.boot=10, size.boot=N
       
       ris<-list(all.selected.psi=drop(all.selected.psi),all.selected.ss=all.selected.ss, all.psi=all.est.psi, all.ss=all.ss)
       
+      #browser()
+      
       if(is.null(o0$obj)){
-        PSI1 <- matrix(rep(est.psi0, rep(nrow(Z), length(est.psi0))), ncol = length(est.psi0))
+        #quando vengono restituiti psi troppo vicini e l'SE non si puo' calcolare, possiamo distanziarli..
+        #Pero' il processo deve essere esteso nel caso in cui ci sono 3 psi vicini..
+        min.n <- opz$min.n-1
+        npsi <- tapply(opz$id.psi.group, opz$id.psi.group, length)
+        nomiAll <- colnames(rangeZ) #rep(opz$nomiSeg, npsi)
+        nomiSeg <- unique(nomiAll)
+        newPsi<-vector("list", length(npsi) )
+        for(.j in 1:length(npsi)){
+          psi.j <- sort(est.psi0[opz$id.psi.group==.j])
+          id  <- nomiSeg[.j]==nomiAll
+          Z.ok <- unique(Z[, id, drop=FALSE][,1])
+          m.j <- min(limZ[1,id])
+          M.j <- max(limZ[2,id])
+          h=1/1.05
+          id.while<-tabulate(cut(Z.ok, c(m.j-10, psi.j, M.j+10), labels=FALSE))<=min.n
+          while(any(id.while)){
+            h<-h*1.05
+            ll <- min(diff(sort(unique(Z.ok))))*h
+            M <- matrix(c(m.j, rep(psi.j, each=2), M.j), ncol=2, byrow=TRUE)
+            Delta <- diff(c(m.j, psi.j, M.j))
+            #id.row <- which.min(Delta)
+            id.row <- which(id.while)
+            id.row <- id.row[which.min(Delta[id.row])]
+            if(id.row<length(Delta)){ #increnmta psi se il "problema" NON riguarda l'ultimo psi
+              psi.j[id.row]<- psi.j[id.row] + abs(min(Delta)-ll)/2
+            } else { #.. altrimenti riducilo
+              psi.j[length(psi.j)]<- psi.j[length(psi.j)] - abs(min(Delta)-ll)/2
+            }
+            id.while<-tabulate(cut(Z.ok, c(m.j-10, psi.j, M.j+10), labels=FALSE))<=min.n
+          }
+          newPsi[[.j]]<-psi.j
+        } #end .j
+        est.psi0 <- unlist(newPsi)
+        PSI1 <- matrix(est.psi0, n, ncol = length(est.psi0), byrow=TRUE)
         o0<-try(suppressWarnings(seg.glm.fit(y, XREG, Z, PSI1, w, offs, opz1)), silent=TRUE)
+        warning("Breakpoint estimates have been outdistanced to allow finite estimates and st.errs", call.=FALSE, immediate.=TRUE)
+        #warning(" 'The final fit (if returned) could be unreliable. Reduce no. of psi or try to increase 'break.boot'", call.=FALSE, immediate.=TRUE)
+        #warning("'Convergence' is suspect: the final fit could be unreliable. Try to re-run by increasing 'break.boot'", call.=FALSE, immediate.=TRUE)
       }
+      
+      
       if(!is.list(o0)) return(0)
       o0$boot.restart<-ris
-      rm(.Random.seed, envir=globalenv())
+      o0$seed <- seed
+      #rm(.Random.seed, envir=globalenv())
       return(o0)
       }
\ No newline at end of file
