@@ -58,12 +58,12 @@ function(obj, seg.Z, psi, npsi, fixed.psi=NULL, control = seg.control(), model =
     fc<- min(max(abs(control$fc),.8),1)
     maxit.glm <- control$maxit.glm
     it.max <- old.it.max<- control$it.max
-    min.step<-control$min.step
+    #min.step<-control$min.step
     alpha<-control$alpha
     digits<-control$digits
     toll <- control$toll
     if(toll<0) stop("Negative tolerance ('tol' in seg.control()) is meaningless", call. = FALSE)
-    visual <- control$visual
+    
     stop.if.error<-control$stop.if.error
     fix.npsi<-fix.npsi<-control$fix.npsi
     if(!is.null(stop.if.error)) {#if the old "stop.if.error" has been used..
@@ -78,21 +78,24 @@ function(obj, seg.Z, psi, npsi, fixed.psi=NULL, control = seg.control(), model =
     random<-control$random
     pow<-control$pow
     conv.psi<-control$conv.psi
+    visual <- control$visual
     visualBoot<-FALSE
-    if(n.boot>0){
-      if(!is.null(control$seed)) {
-        set.seed(control$seed)
-        employed.Random.seed<-control$seed
-      } else {
-        employed.Random.seed<-eval(parse(text=paste(sample(0:9, size=6), collapse="")))
-        set.seed(employed.Random.seed)
-      }
-      if(visual) {visual<-FALSE; visualBoot<-TRUE}#warning("`display' set to FALSE with bootstrap restart", call.=FALSE)}
-      if(!stop.if.error) stop("Bootstrap restart only with a fixed number of breakpoints")
-    }
+    if(visual && n.boot>0) {visual<-FALSE; visualBoot<-TRUE}
+    # if(n.boot>0){
+    #   if(!is.null(control$seed)) {
+    #     set.seed(control$seed)
+    #     employed.Random.seed<-control$seed
+    #   } else {
+    #     employed.Random.seed<-eval(parse(text=paste(sample(0:9, size=6), collapse="")))
+    #     set.seed(employed.Random.seed)
+    #   }
+    #   if(visual) {visual<-FALSE; visualBoot<-TRUE}#warning("`display' set to FALSE with bootstrap restart", call.=FALSE)}
+    #   if(!stop.if.error) stop("Bootstrap restart only with a fixed number of breakpoints")
+    # }
     last <- control$last
     K<-control$K
-    h<-min(abs(control$h),1)
+    #h<-min(abs(control$h),1)
+    h<- control$h
     if(h<1) it.max<-it.max+round(it.max/2)
     orig.call<-Call<-mf<-obj$call
     orig.call$formula<-mf$formula<-formula(obj) #per consentire lm(y~.)
@@ -272,20 +275,27 @@ function(obj, seg.Z, psi, npsi, fixed.psi=NULL, control = seg.control(), model =
     nomiOK<-nomiU
     if(is.null(alpha)) alpha<- max(.05, 1/length(y))
     if(length(alpha)==1) alpha<-c(alpha, 1-alpha)
-    opz<-list(toll=toll, h=h, stop.if.error=stop.if.error, dev0=dev0, visual=visual, it.max=it.max, nomiOK=nomiOK,
-        fam=fam, eta0=obj$linear.predictors, maxit.glm=maxit.glm, id.psi.group=id.psi.group, gap=gap,
-        conv.psi=conv.psi, alpha=alpha, fix.npsi=fix.npsi, min.step=min.step,
-        pow=pow, visualBoot=visualBoot, digits=digits, fc=fc)   
+    eta0<- if(is.null(control$eta)) obj$linear.predictors else control$eta
+    opz<-list(toll=toll, h=h, stop.if.error=stop.if.error, dev0=dev0, visual=visual, it.max=it.max, nomiOK=nomiOK, usesegreg=FALSE,
+        fam=fam, maxit.glm=maxit.glm, id.psi.group=id.psi.group, gap=gap, tol.opt=control$tol.opt,
+        conv.psi=conv.psi, alpha=alpha, fix.npsi=fix.npsi,eta0=eta0, # min.step=min.step,
+        pow=pow, visualBoot=visualBoot, digits=digits, fc=fc, seed=control$seed, fit.psi0=control$fit.psi0, min.n=control$min.n)   
 
+    #browser()
+    
     if(n.boot<=0){
-      obj<-seg.glm.fit(y,XREG,Z,PSI,weights,offs,opz)
+      obj<-seg.glm.fit(y, XREG, Z, PSI, weights, offs, opz)
     } else {
       obj<-seg.glm.fit.boot(y, XREG, Z, PSI, weights, offs, opz, n.boot=n.boot, size.boot=size.boot, random=random, break.boot=break.boot) #jt, nonParam
-      }
+      
+    }
     if(!is.list(obj)){
-        warning("No breakpoint estimated", call. = FALSE)
+        warning("Estimation failed. Too many breakpoints? Returning a glm fit..", call. = FALSE)
         return(obj0)
-        }
+    }
+    seed<- obj$seed
+    
+    #browser()
     
     id.psi.group<-obj$id.psi.group
     nomiOK<-obj$nomiOK
@@ -309,8 +319,10 @@ function(obj, seg.Z, psi, npsi, fixed.psi=NULL, control = seg.control(), model =
 #        if(any(table(sumV)<=1) && stop.if.error) stop("only 1 datum in an interval: breakpoint(s) at the boundary or too close each other")
 #         }
     rangeZ<-obj$rangeZ 
-    obj<-obj$obj
-    beta.c <- coef(obj)[paste("U", 1:ncol(U), sep = "")]
+    idU<-obj$idU
+    obj <- obj$obj
+    beta.c<- obj$coefficients[idU]
+    #beta.c <- coef(obj)[paste("U", 1:ncol(U), sep = "")]
     Vxb <- V %*% diag(beta.c, ncol = length(beta.c))
 
     #psi.values[[length(psi.values) + 1]] <- psi #in LM e' commentata..
@@ -359,7 +371,7 @@ function(obj, seg.Z, psi, npsi, fixed.psi=NULL, control = seg.control(), model =
     if(is.matrix(y)&& (fam$family=="binomial" || fam$family=="quasibinomial")){
               mfExt<-cbind(mfExt[[1]], mfExt[,-1])
     }
-    objF <- update(obj0, formula = Fo, data = mfExt, evaluate=FALSE)
+    objF <- update(obj0, formula = Fo, data = mfExt, family=obj0$family, evaluate=FALSE)
     if(!is.null(objF[["subset"]])) objF[["subset"]]<-NULL
     objF<-eval(objF, envir=mfExt)
     #C'e' un problema..controlla obj (ha due "(Intercepts)" - bhu.. al 27/03/14 non mi sembra!
@@ -387,17 +399,21 @@ function(obj, seg.Z, psi, npsi, fixed.psi=NULL, control = seg.control(), model =
 #------>>>
 #------>>>
 #------>>>
-    if(!gap){
+    #browser()
+    
+    #if(!gap){
         names.coef<-names(objF$coefficients)
-        names(obj$coefficients)[match(c(paste("U",1:k, sep=""), paste("V",1:k, sep="")), names(coef(obj)))]<- nnomi  
-        objF$coefficients[names.coef]<-obj$coefficients[names.coef] #sostituisce gli 0 
+        #names(obj$coefficients)[match(c(paste("U",1:k, sep=""), paste("V",1:k, sep="")), names(coef(obj)))]<- nnomi  
+        #objF$coefficients[names.coef]<-obj$coefficients[names.coef] #sostituisce gli 0 
+        objF$coefficients<-obj$coefficients
+        names(objF$coefficients) <- names.coef
         objF$fitted.values<-obj$fitted.values
         objF$linear.predictors<-obj$linear.predictors
         objF$residuals<-obj$residuals
         objF$deviance<-obj$deviance
         objF$aic<-obj$aic + 2*ncol(Z) #k
         objF$weights<-obj$weights
-    }
+    #}
     
     Cov <- vcov(objF)
     id <- match(nomiVxb, names(coef(objF)))
@@ -427,6 +443,8 @@ function(obj, seg.Z, psi, npsi, fixed.psi=NULL, control = seg.control(), model =
     objF$psi.history <- psi.values
     objF$psi <- ris.psi
     objF$it <- (it - 1)
+    #browser()
+    
     objF$epsilon <- obj$epsilon
     objF$call <- match.call()
     objF$nameUV <- list(U = drop(nomiU), V = rownames(ris.psi), Z = nomiFINALI) #Z = name.Z
@@ -437,7 +455,7 @@ function(obj, seg.Z, psi, npsi, fixed.psi=NULL, control = seg.control(), model =
     ###########################PSI FIXED
     objF$indexU<-build.all.psi(psi.list, fixed.psi)
     if (model)  objF$model <- mf #objF$mframe <- data.frame(as.list(KK))
-    if(n.boot>0) objF$seed<-employed.Random.seed
+    if(n.boot>0) objF$seed<- seed #employed.Random.seed
     objF$psi[,"Initial"]<-NA
     class(objF) <- c("segmented", class(obj0))
     list.obj[[length(list.obj) + 1]] <- objF
