@@ -38,19 +38,38 @@ step.glm.fit.boot <- function(y, XREG, Z, PSI, w, offs, opz, n.boot=10, size.boo
     r
   }
   #-------------
-  #obj<- jump.fit(y, XREG=x.lin, Z=Xtrue, PSI, w=ww, offs, opz, return.all.sol=FALSE)
-  #--------------
+  if(is.null(opz$seed)){
+    mY <- mean(y)
+    sepDec<-if(options()$OutDec==".") "\\." else "\\,"
+    vv <- strsplit(paste(strsplit(paste(mY), sepDec)[[1]], collapse=""),"")[[1]]
+    vv<-vv[vv!="0"]
+    vv=na.omit(vv[1:5])
+    seed <-eval(parse(text=paste(vv, collapse="")))
+    set.seed(seed)
+  } else {
+    if(is.na(opz$seed)) {
+      seed <-eval(parse(text=paste(sample(0:9, size=6), collapse="")))
+      set.seed(seed)
+    } else {
+      seed <-opz$seed
+      set.seed(opz$seed)
+    }
+  }  
   visualBoot<-opz$display
   opz$display<-FALSE
-  opz.boot<-opz
-  opz.boot$pow=c(1,1) #c(1.1,1.2)
-  opz1<-opz
-  opz1$it.max <-1
+  #opz.boot<-opz
+  #opz.boot$pow=c(1,1) #c(1.1,1.2)
+  opz1<-opz #opz1 viene usata solo quando diversi tentativi di stimare il modello falliscono...
+  opz1$it.max <-0
+  opz0 <- opz
+  opz0$maxit.glm <- 2
+  opz0$agg<-.2
   n<-length(y)
   alpha<-opz$alpha
-  limZ <- apply(Z, 2, quantile, names = FALSE, probs = c(alpha, 1 - alpha))
+  limZ <- apply(Z, 2, quantile, names = FALSE, probs = alpha) #c(alpha, 1 - alpha))
   rangeZ <- apply(Z, 2, range) #serve sempre
-  o0 <-try(suppressWarnings(step.glm.fit(y, XREG, Z, PSI, w, offs, opz, return.all.sol=FALSE)), silent=TRUE)
+  o0 <-try(suppressWarnings(step.glm.fit(y, XREG, Z, PSI, w, offs, opz0, return.all.sol=FALSE)), silent=TRUE)
+  #browser()
   if(!is.list(o0)) {
     o0<- suppressWarnings(step.glm.fit(y, XREG, Z, PSI, w, offs, opz, return.all.sol=TRUE))
     o0<-extract.psi(o0)
@@ -60,17 +79,20 @@ step.glm.fit.boot <- function(y, XREG, Z, PSI, w, offs, opz, n.boot=10, size.boo
   if(is.list(o0)){
     est.psi00<-est.psi0<-o0$psi
     ss00<-o0$SumSquares.no.gap
+    eta0 <- o0$eta0
     if(!nonParam) fitted.ok<-fitted(o0)
   } else {
     if(!nonParam) stop("the first fit failed and I cannot extract fitted values for the semipar boot")
     if(random) {
       est.psi00<-est.psi0<-apply(limZ,2,function(r)runif(1,r[1],r[2]))
-      PSI1 <- matrix(rep(est.psi0, rep(nrow(Z), length(est.psi0))), ncol = length(est.psi0))
+      PSI1 <- matrix(est.psi0, n, ncol = length(est.psi0), byrow=TRUE)
       o0<-try(suppressWarnings(step.glm.fit(y, XREG, Z, PSI1, w, offs, opz1)), silent=TRUE)
       ss00<-o0$SumSquares.no.gap
+      eta0 <- o0$eta0
     } else {
       est.psi00<-est.psi0<-apply(PSI,2,mean)
       ss00<-opz$dev0
+      eta0 <- NULL
     }
   }
   
@@ -89,6 +111,7 @@ step.glm.fit.boot <- function(y, XREG, Z, PSI, w, offs, opz, n.boot=10, size.boo
   for(k in seq(n.boot)){
     #if(k==7) browser()
     ##se gli *ultimi* n.boot.rev valori di ss sono uguali, cambia i psi...
+    opz$eta0 <- eta0
     n.boot.rev<- 3 #3 o 4?
     diff.selected.ss <- rev(diff(na.omit(all.selected.ss)))
     if(length(diff.selected.ss)>=(n.boot.rev-1) && all(round(diff.selected.ss[1:(n.boot.rev-1)],6)==0)){
@@ -103,42 +126,49 @@ step.glm.fit.boot <- function(y, XREG, Z, PSI, w, offs, opz, n.boot=10, size.boo
       #est.psi0<- jitter(est.psi0, amount=min(diff(est.psi0))) 
     }
     
-    PSI <- matrix(rep(est.psi0, rep(nrow(Z), length(est.psi0))), ncol = length(est.psi0))
+    PSI <- matrix(est.psi0, n, ncol = length(est.psi0), byrow=TRUE)
     if(jt) Z<-apply(Z.orig,2,jitter)
     if(nonParam){
       id<-sample(n, size=size.boot, replace=TRUE)
 
       o.boot<-try(suppressWarnings(step.glm.fit(y[id], XREG[id,,drop=FALSE], Z[id,,drop=FALSE], PSI[id,,drop=FALSE],
-                                            w[id], offs[id], opz.boot)), silent=TRUE)
+                                            w[id], offs[id], opz)), silent=TRUE)
     } else {
       yy<-fitted.ok+sample(residuals(o0),size=n, replace=TRUE)
-      o.boot<-try(suppressWarnings(step.glm.fit(yy, XREG, Z.orig, PSI, weights, offs, opz.boot)), silent=TRUE)
+      o.boot<-try(suppressWarnings(step.glm.fit(yy, XREG, Z.orig, PSI, weights, offs, opz)), silent=TRUE)
     }
     if(is.list(o.boot)){
       all.est.psi.boot[k,]<-est.psi.boot<-o.boot$psi
     } else {
       est.psi.boot<-apply(limZ,2,function(r)runif(1,r[1],r[2]))
     }
-    PSI <- matrix(rep(est.psi.boot, rep(nrow(Z), length(est.psi.boot))), ncol = length(est.psi.boot))
+    PSI <- matrix(est.psi.boot, n, ncol = length(est.psi.boot), byrow=TRUE)
     #opz$h<-max(opz$h*.9, .2)
     opz$it.max<-opz$it.max+1
     opz$agg<-agg.values[k]
+    #
+    opz$Nboot <- k
+    #
     o <-try(suppressWarnings(step.glm.fit(y, XREG, Z.orig, PSI, w, offs, opz, return.all.sol=TRUE)), silent=TRUE)
     if(!is.list(o) && random){
       est.psi0<-apply(limZ,2,function(r)runif(1,r[1],r[2]))
-      PSI1 <- matrix(rep(est.psi0, rep(nrow(Z), length(est.psi0))), ncol = length(est.psi0))
+      PSI1 <- matrix(est.psi0, n, ncol = length(est.psi0), byrow=TRUE)
       o <-try(suppressWarnings(step.glm.fit(y, XREG, Z, PSI1, w, offs, opz1)), silent=TRUE)
       count.random<-count.random+1
     }
     #se il modello e' stato stimato controlla se la soluzione e' migliore..
     if(is.list(o)){
-      if(!"coefficients"%in%names(o$obj)) o<-extract.psi(o)
-      all.est.psi[k,]<-o$psi
-      all.ss[k]<-o$SumSquares.no.gap
-      if(o$SumSquares.no.gap<=ifelse(is.list(o0), o0$SumSquares.no.gap, 10^12)) o0<-o
-      est.psi0<-o0$psi
-      all.selected.psi[k,] <- est.psi0
-      all.selected.ss[k]<-o0$SumSquares.no.gap #min(c(o$SumSquares.no.gap, o0$SumSquares.no.gap))
+      if(!"coefficients"%in%names(o$obj)) o<-suppressWarnings(try(extract.psi(o), silent=TRUE))
+      #if(class(o)!="try-error"){
+      if(!inherits(o, "try-error")){
+        all.est.psi[k,]<-o$psi
+        all.ss[k]<-o$SumSquares.no.gap
+        if(o$SumSquares.no.gap<=ifelse(is.list(o0), o0$SumSquares.no.gap, 10^12)) o0<-o
+        est.psi0<-o0$psi
+        all.selected.psi[k,] <- est.psi0
+        all.selected.ss[k]<-L0<-o0$SumSquares.no.gap
+        eta0 <- o0$eta0
+      }
     }
     
     
@@ -180,11 +210,13 @@ step.glm.fit.boot <- function(y, XREG, Z, PSI, w, offs, opz, n.boot=10, size.boo
   ris<-list(all.selected.psi=drop(all.selected.psi),all.selected.ss=all.selected.ss, all.psi=all.est.psi, all.ss=all.ss)
   
   if(is.null(o0$obj)){
-    PSI1 <- matrix(rep(est.psi0, rep(nrow(Z), length(est.psi0))), ncol = length(est.psi0))
+    PSI1 <- matrix(est.psi0, n, ncol = length(est.psi0),byrow=TRUE)
     o0 <- try(step.glm.fit(y, XREG, Z, PSI1, w, offs, opz1), silent=TRUE)
+    warning("The final fit can be unreliable (possibly mispecified stepmented relationship)", call.=FALSE, immediate.=TRUE)
   }
   if(!is.list(o0)) return(0)
   o0$boot.restart<-ris
-  rm(.Random.seed, envir=globalenv())
+  o0$seed <- seed
+  #rm(.Random.seed, envir=globalenv())
   return(o0)
 }
