@@ -36,20 +36,45 @@ step.ts.fit.boot <- function(y, XREG, Z, PSI, opz, n.boot=10, size.boot=NULL, jt
     r<-list(SumSquares.no.gap=dev.ok, psi=psi.ok)
     r
   }
+  
+  #browser()
+  
+  if(is.null(opz$seed)){
+    mY <- mean(y)
+    sepDec<-if(options()$OutDec==".") "\\." else "\\,"
+    vv <- strsplit(paste(strsplit(paste(mY), sepDec)[[1]], collapse=""),"")[[1]]
+    vv<-vv[vv!="0"]
+    vv=na.omit(vv[1:5])
+    seed <-eval(parse(text=paste(vv, collapse="")))
+    set.seed(seed)
+  } else {
+    if(is.na(opz$seed)) {
+      seed <-eval(parse(text=paste(sample(0:9, size=6), collapse="")))
+      set.seed(seed)
+    } else {
+      seed <-opz$seed
+      set.seed(opz$seed)
+    }
+  }  
+  
   #-------------
   #obj<- jump.fit(y, XREG=x.lin, Z=Xtrue, PSI, w=ww, offs, opz, return.all.sol=FALSE)
   #--------------
   visualBoot<-opz$display
   opz$display<-FALSE
-  opz.boot<-opz
-  opz.boot$pow=c(1,1) #c(1.1,1.2)
+  #opz.boot<-opz
+  #opz.boot$pow=c(1,1) #c(1.1,1.2)
   opz1<-opz
-  opz1$it.max <-1
+  opz1$it.max <- 0
+  opz0<-opz
+  opz0$agg<-.2
   n<-length(y)
   rangeZ <- apply(Z, 2, range) #serve sempre
   alpha <- opz$alpha
-  limZ <- apply(Z, 2, quantile, names = FALSE, probs = c(alpha, 1 - alpha))
-  o0 <-try(suppressWarnings(step.ts.fit(y, XREG, Z, PSI, opz, return.all.sol=FALSE)), silent=TRUE)  
+  limZ <- apply(Z, 2, quantile, names = FALSE, probs = alpha)
+  o0 <-try(suppressWarnings(step.ts.fit(y, XREG, Z, PSI, opz0, return.all.sol=FALSE)), silent=TRUE)  
+  #browser()
+  
   if(!is.list(o0)) {
     o0<- suppressWarnings(step.ts.fit(y, XREG, Z, PSI, opz, return.all.sol=TRUE))
     o0<-extract.psi(o0)
@@ -64,7 +89,7 @@ step.ts.fit.boot <- function(y, XREG, Z, PSI, opz, n.boot=10, size.boot=NULL, jt
     if(!nonParam) stop("the first fit failed and I cannot extract fitted values for the semipar boot")
     if(random) {
       est.psi00<-est.psi0<-apply(limZ,2,function(r)runif(1,r[1],r[2]))
-      PSI1 <- matrix(rep(est.psi0, rep(nrow(Z), length(est.psi0))), ncol = length(est.psi0))
+      PSI1 <- matrix(est.psi0, n, ncol = length(est.psi0), byrow=TRUE)
       o0<-try(suppressWarnings(step.ts.fit(y, XREG, Z, PSI1, opz1)), silent=TRUE)
       ss00<-o0$SumSquares.no.gap
     } else {
@@ -104,22 +129,22 @@ step.ts.fit.boot <- function(y, XREG, Z, PSI, opz, n.boot=10, size.boot=NULL, jt
       #est.psi0<- jitter(est.psi0, amount=min(diff(est.psi0))) 
     }
     
-    PSI <- matrix(rep(est.psi0, rep(nrow(Z), length(est.psi0))), ncol = length(est.psi0))
+    PSI <- matrix(est.psi0, n, ncol = length(est.psi0), byrow=TRUE)
     if(jt) Z<-apply(Z.orig,2,jitter)
     if(nonParam){
       id<-sample(n, size=size.boot, replace=TRUE)
       o.boot<-try(suppressWarnings(step.ts.fit(y[id], XREG[id,,drop=FALSE], Z[id,,drop=FALSE], PSI[id,,drop=FALSE],
-                                             opz.boot)), silent=TRUE)
+                                             opz)), silent=TRUE)
     } else {
       yy<-fitted.ok+sample(residuals(o0),size=n, replace=TRUE)
-      o.boot<-try(suppressWarnings(step.ts.fit(yy, XREG, Z.orig, PSI, opz.boot)), silent=TRUE)
+      o.boot<-try(suppressWarnings(step.ts.fit(yy, XREG, Z.orig, PSI, opz)), silent=TRUE)
     }
     if(is.list(o.boot)){
       all.est.psi.boot[k,]<-est.psi.boot<-o.boot$psi
     } else {
       est.psi.boot<-apply(limZ,2,function(r)runif(1,r[1],r[2]))
     }
-    PSI <- matrix(rep(est.psi.boot, rep(nrow(Z), length(est.psi.boot))), ncol = length(est.psi.boot))
+    PSI <- matrix(est.psi.boot, n, ncol = length(est.psi.boot), byrow=TRUE)
     #opz$h<-max(opz$h*.9, .2)
     opz$it.max<-opz$it.max+1
     opz$agg<-agg.values[k]
@@ -179,11 +204,13 @@ step.ts.fit.boot <- function(y, XREG, Z, PSI, opz, n.boot=10, size.boot=NULL, jt
   ris<-list(all.selected.psi=drop(all.selected.psi),all.selected.ss=all.selected.ss, all.psi=all.est.psi, all.ss=all.ss)
   
   if(is.null(o0$obj)){
-    PSI1 <- matrix(rep(est.psi0, rep(nrow(Z), length(est.psi0))), ncol = length(est.psi0))
+    PSI1 <- matrix(est.psi0, n, ncol = length(est.psi0), byrow=TRUE)
     o0 <- try(step.ts.fit(y, XREG, Z, PSI1, opz1), silent=TRUE)
+    warning("The final fit can be unreliable (possibly mispecified segmented relationship)", call.=FALSE, immediate.=TRUE)
   }
   if(!is.list(o0)) return(0)
   o0$boot.restart<-ris
-  rm(.Random.seed, envir=globalenv())
+  o0$seed<-seed
+  #rm(.Random.seed, envir=globalenv())
   return(o0)
 }
