@@ -1,5 +1,5 @@
 stepmented.glm <- function(obj, seg.Z, psi, npsi, fixed.psi=NULL, 
-                           control=seg.control(), keep.class=FALSE, var.psi=TRUE, ...) {
+                           control=seg.control(), keep.class=FALSE, var.psi=FALSE, ...) {
   # ---------
   toMatrix<-function(.x, ki){
     # ripete ogni .x[,j] ki[j] volte
@@ -13,17 +13,18 @@ stepmented.glm <- function(obj, seg.Z, psi, npsi, fixed.psi=NULL,
   agg<- 1-control$fc
   maxit.glm <- control$maxit.glm
   it.max<- control$it.max 
-  tol<-  control$tol
+  tol<-  control$toll
   display<- control$visual 
   digits <- control$digits 
-  min.step <- control$min.step
-  conv.psi <- control$conv.psi 
+  #min.step <- control$min.step
+  #conv.psi <- control$conv.psi 
   alpha <- control$alpha
   fix.npsi <- control$fix.npsi 
   n.boot <- control$n.boot 
   break.boot<- control$break.boot +2 
   seed<- control$seed
   fix.npsi<-control$fix.npsi
+  h<- control$h
   #-----------
   
   if(!(inherits(obj,"glm") || is.vector(obj) || is.ts(obj))) stop("obj should be a 'glm' fit, a 'vector' or 'ts' object")
@@ -31,7 +32,7 @@ stepmented.glm <- function(obj, seg.Z, psi, npsi, fixed.psi=NULL,
   if(is.vector(obj) || is.ts(obj)){
     #if(is.matrix(obj) && ncol(obj)>1) stop("if matrix 'obj' should have 1 column")
     #obj<-drop(obj)
-    if(!missing(seg.Z) && length(all.vars(seg.Z))>1) stop(" multiple seg.Z allowed only with lm models")
+    if(!missing(seg.Z) && length(all.vars(seg.Z))>1) stop(" multiple seg.Z allowed only with (g)lm models")
     Fo0<-as.formula(paste(deparse(substitute(obj))," ~ 1", sep=""))
     y.only.vector<-TRUE
     
@@ -66,7 +67,7 @@ stepmented.glm <- function(obj, seg.Z, psi, npsi, fixed.psi=NULL,
     orig.call<-NULL
     fam<- list(...)$family
     if(!(class(fam)[1]=="family" && is.list(fam))) stop(" family in ... should be specified like 'gaussian()'")
-    dev0<-100
+    dev0<-1e+6
     ####################################################
   } else { #se obj e' un glm
     y.only.vector <- FALSE
@@ -182,7 +183,7 @@ stepmented.glm <- function(obj, seg.Z, psi, npsi, fixed.psi=NULL,
     XREGseg<-XREG[,id.n.Seg,drop=FALSE]
     XREG <- XREG[, match(c("(Intercept)", namesXREG0),colnames(XREG), nomatch = 0), drop = FALSE]
     XREG<-XREG[,unique(colnames(XREG)), drop=FALSE]
-    n <- nrow(XREG)
+    #n <- nrow(XREG)
     
     #browser()
 
@@ -193,7 +194,7 @@ stepmented.glm <- function(obj, seg.Z, psi, npsi, fixed.psi=NULL,
       psi <- list(as.numeric(psi))
       names(psi)<-name.Z
     }
-    if (!is.list(Z) || !is.list(psi) || is.null(names(Z)) || is.null(names(psi))) stop("'psi' has to be *named* list")
+    if (!is.list(Z) || !is.list(psi) || is.null(names(Z)) || is.null(names(psi))) stop("'psi' or 'npsi' have to be *named* when there are multiple stepmented variables")
     id.nomiZpsi <- match(names(Z), names(psi))
     if ((length(Z)!=length(psi)) || any(is.na(id.nomiZpsi))) stop("Length or names of 'seg.Z' and 'psi' do not match")
     nome <- names(psi)[id.nomiZpsi]
@@ -307,26 +308,29 @@ stepmented.glm <- function(obj, seg.Z, psi, npsi, fixed.psi=NULL,
   if(sum(c1 + c2) != 0 || is.na(sum(c1 + c2)) ) stop("starting psi out of the admissible range")
   if(is.null(alpha)) alpha<- max(.05, 1/length(y))
   if(length(alpha)==1) alpha<-c(alpha, 1-alpha)
-  
-  opz<-list(toll=tol, dev0=dev0, display=display, it.max=it.max, agg=agg, digits=digits, rangeZ=rangeZ,
-            fam=fam, maxit.glm=maxit.glm,
-            #nomiOK=nomiOK, id.psi.group=id.psi.group, visualBoot=visualBoot, invXtX=invXtX, Xty=Xty, 
-            conv.psi=conv.psi, alpha=alpha, fix.npsi=fix.npsi, min.step=min.step, npsii=npsii) #, npsii=npsii, P=P)
-  
+  eta0 <- if(is.null(control$eta)) obj$linear.predictors else control$eta
+  opz<-list(toll=tol, dev0=dev0, display=display, it.max=it.max, agg=agg, digits=digits, rangeZ=rangeZ, usestepreg=FALSE,
+            fam=fam, maxit.glm=maxit.glm, id.psi.group=id.psi.group, h=h, eta0=eta0,
+            #nomiOK=nomiOK,  visualBoot=visualBoot, invXtX=invXtX, Xty=Xty, conv.psi=conv.psi,  min.step=min.step,
+            alpha=alpha, fix.npsi=fix.npsi, npsii=npsii, seed=control$seed, fit.psi0=control$fit.psi0)
+  opz$Nboot <- 0
   # #################################################################################
   # #### jump.fit(y, XREG=x.lin, Z=Xtrue, PSI, w=ww, offs, opz, return.all.sol=FALSE)
   # #################################################################################
   if(n.boot<=0){
     obj<- step.glm.fit(y, x.lin, Xtrue, PSI, ww, offs, opz, return.all.sol=FALSE)
   } else {
-    if("seed" %in% names(control)) set.seed(control$seed)
-    obj<-step.glm.fit.boot(y, x.lin, Xtrue, PSI, ww, offs, opz, n.boot, break.boot=break.boot) #, size.boot=size.boot, random=random)
+    #if("seed" %in% names(control)) set.seed(control$seed)
+    obj<-step.glm.fit.boot(y, x.lin, Xtrue, PSI, ww, offs, opz, n.boot, break.boot=break.boot) 
+    seed <- control$seed
   }
   # if(!is.list(obj)){
   #   warning("No breakpoint estimated", call. = FALSE)
   #   return(obj0)
   # }
 
+  #browser()
+  
   id.warn<-obj$id.warn
   it<-obj$it
   psi<-obj$psi
@@ -478,6 +482,10 @@ stepmented.glm <- function(obj, seg.Z, psi, npsi, fixed.psi=NULL,
   X <- cbind(x.lin,U)
   #objF$obj.ok<-mylm(X, y, w=ww, offs=offs) #coefficients=b,fitted.values=fit,residuals=r, df.residual=length(y)-length(b))
   objF$obj.ok<-glm.fit(X, y, weights =ww, offset =offs, family=fam) #, control = glm.control(maxit = maxit.glm), etastart = eta0)
+  
+  #browser()
+  objF$obj.ok$invXtX <- chol2inv(qr.R(objF$obj.ok$qr))
+  
   objF$objW<- objW
   objF$fitted.values<-objF$obj.ok$fitted.values
   objF$residuals<- objF$obj.ok$residuals
@@ -501,7 +509,7 @@ stepmented.glm <- function(obj, seg.Z, psi, npsi, fixed.psi=NULL,
   objF$it <- it 
   objF$epsilon <- obj$epsilon
   objF$id.warn <- id.warn
-  
+  if(n.boot>0) objF$seed <- seed
   #browser()
   
   #class(objF) <- if(y.only.vector) "stepmented" else c("stepmented", class(obj0))
@@ -543,7 +551,6 @@ stepmented.glm <- function(obj, seg.Z, psi, npsi, fixed.psi=NULL,
     objF$psi[,"St.Err"]<-sqrt(vv)
     objF$vcov<- Cov
   }
-  
   class(objF) <- c("stepmented", class(objF))
   return(objF)
 }
