@@ -1,5 +1,5 @@
 stepmented.lm <- function(obj, seg.Z, psi, npsi, fixed.psi=NULL, control=seg.control(), 
-                          keep.class=FALSE, var.psi=TRUE, ...) {
+                          keep.class=FALSE, var.psi=FALSE, ...) {
   # ---------
   mylm<-function(x,y,w=1,offs=0){
     x1<-x*sqrt(w)
@@ -9,7 +9,7 @@ stepmented.lm <- function(obj, seg.Z, psi, npsi, fixed.psi=NULL, control=seg.con
     b<-drop(solve(XtX,crossprod(x1,y1)))
     fit<-drop(tcrossprod(x,t(b)))
     r<-y-fit
-    o<-list(coefficients=b,fitted.values=fit,residuals=r, df.residual=length(y)-length(b), XtX=XtX, w=w)
+    o<-list(coefficients=b,fitted.values=fit,residuals=r, df.residual=length(y)-length(b), invXtX=solve(XtX), w=w)
     o
   }
   #-----------
@@ -24,21 +24,23 @@ stepmented.lm <- function(obj, seg.Z, psi, npsi, fixed.psi=NULL, control=seg.con
   #-----------
   agg<- 1-control$fc
   it.max<- control$it.max 
-  tol<-  control$tol
+  tol<-  control$toll
   display<- control$visual 
   digits <- control$digits 
   min.step <- control$min.step
-  conv.psi <- control$conv.psi 
+  #conv.psi <- control$conv.psi 
   alpha <- control$alpha
   fix.npsi <- control$fix.npsi 
   n.boot <- control$n.boot 
   break.boot<- control$break.boot + 2 
   seed<- control$seed
   fix.npsi<-control$fix.npsi
+  h<-control$h
   #-----------
   #browser()
   
-  if(!(inherits(obj,"lm") || is.vector(obj) || is.ts(obj))) stop("obj should be a 'lm' fit, a 'vector' or 'ts' object")
+  #if(!(inherits(obj,"lm") || is.vector(obj) || is.ts(obj))) stop("obj should be a 'lm' fit, a 'vector' or 'ts' object")
+  if(!inherits(obj,"lm")) stop("obj should be a 'lm' fit")
   
     y.only.vector <- FALSE
     Fo0 <- formula(obj) 
@@ -128,6 +130,8 @@ stepmented.lm <- function(obj, seg.Z, psi, npsi, fixed.psi=NULL, control=seg.con
     mfExt<-eval(mfExt, parent.frame())
     
     #mf <- mfExt
+    #browser()
+    if(nrow(mf)!=nrow(mfExt)) stop("missing values in any stepmented covariate?")
     
     
     ww <- as.vector(model.weights(mf))
@@ -163,7 +167,7 @@ stepmented.lm <- function(obj, seg.Z, psi, npsi, fixed.psi=NULL, control=seg.con
       psi <- list(as.numeric(psi))
       names(psi)<-name.Z
     }
-    if (!is.list(Z) || !is.list(psi) || is.null(names(Z)) || is.null(names(psi))) stop("'psi' has to be *named* list")
+    if (!is.list(Z) || !is.list(psi) || is.null(names(Z)) || is.null(names(psi))) stop("'psi' or 'npsi' have to be *named* when there are multiple stepmented variables")
     id.nomiZpsi <- match(names(Z), names(psi))
     if ((length(Z)!=length(psi)) || any(is.na(id.nomiZpsi))) stop("Length or names of 'seg.Z' and 'psi' do not match")
     nome <- names(psi)[id.nomiZpsi]
@@ -230,7 +234,7 @@ stepmented.lm <- function(obj, seg.Z, psi, npsi, fixed.psi=NULL, control=seg.con
     nomiV <- paste(nomiV, nomiZ, sep = ".")
     initial <- psi
     obj0 <- obj
-    dev0 <-sum(obj$residuals^2)
+    dev0 <-sum(ww*obj$residuals^2)
     list.obj <- list(obj)
     nomiOK<-nomiU
   
@@ -251,7 +255,7 @@ stepmented.lm <- function(obj, seg.Z, psi, npsi, fixed.psi=NULL, control=seg.con
   P<-n.Seg
   npsii<-a
   npsi<- sum(npsii)
-  Xtrue<-Z
+  #Xtrue<-Z
   #psi0 <- unlist(psi)
   #PSI<- matrix(psi0, n, npsi, byrow=TRUE)
   #if(ncol(x)!=P) stop("errore")
@@ -260,7 +264,7 @@ stepmented.lm <- function(obj, seg.Z, psi, npsi, fixed.psi=NULL, control=seg.con
   #browser()
   
   if(it.max == 0) {
-    U <- (Xtrue>PSI)
+    U <- (Z>PSI)
     colnames(U) <- paste(ripetizioni, nomiZ, sep = ".")
     nomiU <- paste("U", colnames(U), sep = "")
     #for (i in 1:ncol(U)) assign(nomiU[i], U[, i], envir = KK)
@@ -280,25 +284,30 @@ stepmented.lm <- function(obj, seg.Z, psi, npsi, fixed.psi=NULL, control=seg.con
   }
   
   
-  c1 <- apply((Xtrue <= PSI), 2, all) #dovrebbero essere tutti FALSE (prima era solo <)
-  c2 <- apply((Xtrue >= PSI), 2, all) #dovrebbero essere tutti FALSE (prima era solo >)
+  c1 <- apply((Z <= PSI), 2, all) #dovrebbero essere tutti FALSE (prima era solo <)
+  c2 <- apply((Z >= PSI), 2, all) #dovrebbero essere tutti FALSE (prima era solo >)
   if(sum(c1 + c2) != 0 || is.na(sum(c1 + c2)) ) stop("starting psi out of the admissible range")
   if(is.null(alpha)) alpha<- max(.05, 1/length(y))
   if(length(alpha)==1) alpha<-c(alpha, 1-alpha)
   
-  opz<-list(toll=tol, dev0=dev0, display=display, it.max=it.max, agg=agg, digits=digits, rangeZ=rangeZ,
-            #nomiOK=nomiOK, id.psi.group=id.psi.group, visualBoot=visualBoot, invXtX=invXtX, Xty=Xty, 
-            conv.psi=conv.psi, alpha=alpha, fix.npsi=fix.npsi, min.step=min.step, npsii=npsii) #, npsii=npsii, P=P)
+  #browser()
+  
+  opz<-list(toll=tol, dev0=dev0, display=display, it.max=it.max, agg=agg, digits=digits, rangeZ=rangeZ, usestepreg=FALSE,
+            id.psi.group=id.psi.group, h=h,
+            #nomiOK=nomiOK, , visualBoot=visualBoot, invXtX=invXtX, Xty=Xty, conv.psi=conv.psi, 
+            alpha=alpha, fix.npsi=fix.npsi, min.step=min.step, npsii=npsii, 
+            seed=control$seed, fit.psi0=control$fit.psi0)
   
   # #################################################################################
   # #### jump.fit(y, XREG=x.lin, Z=Xtrue, PSI, w=ww, offs, opz, return.all.sol=FALSE)
   # #################################################################################
   if(n.boot<=0){
-    obj<- step.lm.fit(y, x.lin, Xtrue, PSI, ww, offs, opz, return.all.sol=FALSE)
+    obj<- step.lm.fit(y, x.lin, Z, PSI, ww, offs, opz, return.all.sol=FALSE)
   } else {
     #browser()
-    if("seed" %in% names(control)) set.seed(control$seed)
-    obj<-step.lm.fit.boot(y, x.lin, Xtrue, PSI, ww, offs, opz, n.boot, break.boot=break.boot) #, size.boot=size.boot, random=random)
+    #if("seed" %in% names(control)) set.seed(control$seed)
+    obj<-step.lm.fit.boot(y, x.lin, Z, PSI, ww, offs, opz, n.boot, break.boot=break.boot) 
+    seed<- obj$seed
   }
   # if(!is.list(obj)){
   #   warning("No breakpoint estimated", call. = FALSE)
@@ -311,8 +320,10 @@ stepmented.lm <- function(obj, seg.Z, psi, npsi, fixed.psi=NULL, control=seg.con
   psi<-obj$psi
   psi.values<-if(n.boot<=0) obj$psi.values else obj$boot.restart
   #i beta.c corripondono ai psi NON ordinati!!!
-  beta.c<- obj$beta.c
-  beta.c<-unlist(tapply(psi, id.psi.group, function(.x)beta.c[order(.x)]))
+  #
+  ##Nelle funzioni step i beta.c NON servono. Righe sotto commentate l'8/3/24
+  #beta.c<- obj$beta.c
+  #beta.c<-unlist(tapply(psi, id.psi.group, function(.x)beta.c[order(.x)]))
   #unlist(lapply(unique(id.psi.group), function(.x) beta.c[id.psi.group==.x][order(psi[id.psi.group==.x])]))
   psi<-unlist(tapply(psi, id.psi.group, sort)) 
   Z0<-apply(Z,2,sort)
@@ -325,7 +336,7 @@ stepmented.lm <- function(obj, seg.Z, psi, npsi, fixed.psi=NULL, control=seg.con
   PSI.mid<- matrix(psi, n, npsi, byrow = TRUE)
   
   #bisogna evitare che una qualche x_i sia uguale a psi, altrimenti la costruzione di V-> INF
-  DEN <- abs(Xtrue - PSI.mid)
+  DEN <- abs(Z - PSI.mid)
   DEN <- apply(DEN, 2, function(.x) pmax(.x, sort(.x)[2]/2))  #pmax(.x, diff(range(.x))/1000)) 
   
   #xx=Xtrue - PSI.mid
@@ -334,11 +345,13 @@ stepmented.lm <- function(obj, seg.Z, psi, npsi, fixed.psi=NULL, control=seg.con
   
   V <- (1/(2 * DEN))
   colnames(V)<-nomiV
-  U <- (Xtrue * V + 1/2)
+  U <- (Z * V + 1/2)
   colnames(U)<-nomiU
   Vxb <- -V  #* rep(-beta.c, each = nrow(V))
   nomiVxb <- gsub("V", "psi", nomiV)
   nnomi <- c(nomiU, nomiVxb)
+  #browser()
+  
   for(i in 1:ncol(U)) {
       mfExt[nomiU[i]]<-mf[nomiU[i]] <- U[,i]
       mfExt[nomiVxb[i]]<-mf[nomiVxb[i]] <- Vxb[,i]
@@ -376,7 +389,7 @@ stepmented.lm <- function(obj, seg.Z, psi, npsi, fixed.psi=NULL, control=seg.con
         idNA.psi <- match(nameNA.psi, nomiVxb)
         nomiVxb <- setdiff(nomiVxb, nameNA.psi)
         nomiU <- setdiff(nomiU, nameNA.U)
-        Xtrue <- Xtrue[, -idNA.psi, drop = FALSE]
+        Z <- Z[, -idNA.psi, drop = FALSE]
         PSI.mid<- PSI.mid[, -idNA.psi, drop = FALSE]
         id.psi.group <- id.psi.group[-idNA.psi]
         psi <- psi[-idNA.psi]
@@ -392,7 +405,7 @@ stepmented.lm <- function(obj, seg.Z, psi, npsi, fixed.psi=NULL, control=seg.con
   #browser()
   
   
-  ris.psi<-matrix(NA,length(psi),3)
+  ris.psi<-matrix(NA,length(psi), 3)
   colnames(ris.psi) <- c("Initial", "Est.", "St.Err")
   rownames(ris.psi) <- nomiVxb
   ris.psi[,2]<-psi
@@ -424,9 +437,9 @@ stepmented.lm <- function(obj, seg.Z, psi, npsi, fixed.psi=NULL, control=seg.con
   objF$psi.rounded <- psi.rounded
   #objW<-objF
   #stima il modello "vero" (non-working)
-  U<- (Xtrue > PSI.mid)
+  U<- (Z > PSI.mid)
   colnames(U)<-nomiU
-  X <- cbind(x.lin,U)
+  X <- cbind(x.lin, U)
   objF$objW<- objW
   objF$obj.ok<-mylm(X, y, w=ww, offs=offs) #coefficients=b,fitted.values=fit,residuals=r, df.residual=length(y)-length(b))
   objF$fitted.values<-objF$obj.ok$fitted.values
@@ -442,38 +455,32 @@ stepmented.lm <- function(obj, seg.Z, psi, npsi, fixed.psi=NULL, control=seg.con
   objF$it <- it 
   objF$epsilon <- obj$epsilon
   objF$id.warn <- id.warn
+  if(n.boot>0) objF$seed <- seed
   class(objF) <- c("stepmented", class(obj0))
-  #browser()
-  #Un effetto aggiuntivo..
-    Z.in.obj<-intersect(all.vars(Fo0), all.vars(seg.Z))
-    if(length(Z.in.obj)>0){
-      tt<-terms(Fo0)#, specials=Z.in.obj)
-      #id<-match(Z.in.obj, all.vars(Fo0))-1 #1 e' per la risposta..
-      
-      id<-match(Z.in.obj, intersect(all.vars(Fo0), names(mf)))-1
-      
-      nome<-attr(tt,"term.labels")[id]
-      Fo.ok<-as.formula(paste("~0", nome, sep="+"))
-      f.x<-matrix(NA, 150, ncol(objF$Z[,Z.in.obj,drop=FALSE])) #prima era nrow(objF$Z) invece che 100
-      for(j in 1:length(Z.in.obj)){
-        #browser()
-        #dd<-data.frame(objF$Z[,j])
-        idPsi <- nomiVxb[endsWith(nomiVxb, paste(".", Z.in.obj[j], sep = ""))]
-        psi <- coef(objF)[idPsi]
-        #dd<-data.frame(sort(c(psi, seq(min(objF$Z[,j]), max(objF$Z[,j]), l=nrow(f.x)))))
-        
-        dd<-data.frame(seq(min(objF$Z[,Z.in.obj[j]]), max(objF$Z[,Z.in.obj[j]]), l=nrow(f.x)))
-        names(dd)<- Z.in.obj[j]
-        M<-model.matrix(Fo.ok, data=dd)
-        #M<-M[seq(ceiling(length(psi)/2),by=1,l=nrow(f.x)),,drop=FALSE]
-        f.x[,j]<-M%*% coef(objF)[colnames(M)]
-      }
-      colnames(f.x)<-Z.in.obj
-      objF$f.x<-f.x
-    }
-  objF$psi<- objF$psi[,-1,drop=FALSE] #rimuovi la colonna Initial
   
-  #browser()
+  #Un effetto aggiuntivo..
+  Z.in.obj<-intersect(all.vars(Fo0), all.vars(seg.Z))
+  if(length(Z.in.obj)>0){
+    tt<-terms(Fo0)#, specials=Z.in.obj)
+    #id<-match(Z.in.obj, all.vars(Fo0))-1 #1 e' per la risposta..
+    id<-match(Z.in.obj, intersect(all.vars(Fo0), names(mf)))-1
+      
+    nome<-attr(tt,"term.labels")[id]
+    Fo.ok<-as.formula(paste("~0", nome, sep="+"))
+    f.x<-matrix(NA, 150, ncol(objF$Z[,Z.in.obj,drop=FALSE])) #prima era nrow(objF$Z) invece che 100
+    for(j in 1:length(Z.in.obj)){
+      idPsi <- nomiVxb[endsWith(nomiVxb, paste(".", Z.in.obj[j], sep = ""))]
+      #psi <- coef(objF)[idPsi]
+      dd<-data.frame(seq(min(objF$Z[,Z.in.obj[j]]), max(objF$Z[,Z.in.obj[j]]), l=nrow(f.x)))
+      names(dd)<- Z.in.obj[j]
+      M<-model.matrix(Fo.ok, data=dd)
+      f.x[,j]<-M%*% coef(objF)[colnames(M)]
+    }
+    colnames(f.x)<-Z.in.obj
+    objF$f.x<-f.x
+  }
+  
+  objF$psi<- objF$psi[,-1,drop=FALSE] #rimuovi la colonna Initial
   
   if(var.psi){
     Cov <- vcov.stepmented(objF, k=NULL)
@@ -483,8 +490,7 @@ stepmented.lm <- function(obj, seg.Z, psi, npsi, fixed.psi=NULL, control=seg.con
     objF$vcov<- Cov
   }
   #Cov[nomiVxb, ]<- Cov[, nomiVxb] <- 0
-  
-  
+
   # var.Tay<-function(est1,est2,v1,v2,v12){
   #   r<- est1/est2
   #   vv<-(v1+v2*r^2-2*r*v12)/est2^2
@@ -506,6 +512,5 @@ stepmented.lm <- function(obj, seg.Z, psi, npsi, fixed.psi=NULL, control=seg.con
   # }
   # names(varPsi) <- nomiVxb
 
-  
   return(objF)
 }
